@@ -1,4 +1,5 @@
 #!/usr/bin/env bash
+set -e
 #bessie is a 32-bit Rasberry Pi managed by Bruce Rindahl
 ## BRANCH is passed in via jenkins which is set via gitea web hook
 #export BRANCH=618a67b1d6fc223dd5a4c0b02c824939f21dbd65
@@ -22,10 +23,14 @@ export err_status=0
 make check RUNTESTFLAGS="-v"
 make install
 make check RUNTESTFLAGS="-v --extension"
-err_status=$?
+if [ "$?" != "0" ]; then
+	err_status=$?
+fi
 
 make garden
-err_status=$?
+if [ "$?" != "0" ]; then
+	err_status=$?
+fi
 
 if [ -d $PGDATA/postmaster.pid ] ; then
 	$PGCTL stop -D $PGDATA -s -m fast
