@@ -118,8 +118,10 @@ fi
 
 # Test all available upgrades
 # TODO: protect via some variable ?
-utils/check_all_upgrades.sh \
-        `grep '^POSTGIS_' Version.config | cut -d= -f2 | paste -sd '.'`
-if [ "$?" != "0" ]; then
-  exit $?
+if [ "$PG_VER" < "14"]; then
+  utils/check_all_upgrades.sh \
+          `grep '^POSTGIS_' Version.config | cut -d= -f2 | paste -sd '.'`
+  if [ "$?" != "0" ]; then
+    exit $?
+  fi
 fi
