@@ -573,6 +573,69 @@ static void test_raster_pixel_as_polygon() {
 	cu_free_raster(rast);
 }
 
+
+
+
+static void test_raster_get_pixel_bilinear() {
+	uint32_t width = 2;
+	uint32_t height = 2;
+	double ul_x = 0.0;
+	double ul_y = 0.0;
+	double scale_x = 1;
+	double scale_y = 1;
+
+	double xr, yr;
+	double igt[6];
+
+	rt_raster rast = rt_raster_new(width, height);
+	rt_raster_set_offsets(rast, ul_x, ul_y);
+	rt_raster_set_scale(rast, scale_x, scale_y);
+
+	double xw = 1.5, yw = 0.5;
+
+	rt_raster_generate_new_band(
+		rast,    // rt_raster raster,
+		PT_64BF, // rt_pixtype pixtype,
+		1.0,    // double initialvalue,
+		1,       // uint32_t hasnodata,
+		-99.0,   // double nodatavalue,
+		0        // int index
+	);
+
+	rt_raster_geopoint_to_rasterpoint(
+		rast,
+		xw, yw,
+		&xr, &yr, igt);
+
+	printf("xw = %g, yw = %g, xr = %g, yr = %g\n", xw, yw, xr, yr);
+
+	// err = rt_raster_cell_to_geopoint(
+	// 	rast,
+	// 	xr, yr,
+	// 	&xw, &yw, igt);
+
+	// printf("xw = %g, yw = %g, xr = %g, yr = %g\n", xw, yw, xr, yr);
+
+	rt_band band = rt_raster_get_band(rast, 0);
+	rt_band_set_pixel(band, 0, 0, 10.0, NULL);
+	rt_band_set_pixel(band, 0, 1, 10.0, NULL);
+	rt_band_set_pixel(band, 1, 0, 20.0, NULL);
+	rt_band_set_pixel(band, 1, 1, 40.0, NULL);
+
+
+	double value;
+	int nodata;
+	rt_band_get_pixel_bilinear(
+		band,
+		xw, yw, // double xw, double yw,
+		&value, &nodata // double *r_value, int *r_nodata)
+		);
+
+	printf("xw = %g, yw = %g, value = %g, nodata = %d\n", xr, yr, value, nodata);
+
+
+}
+
 /* register tests */
 void raster_geometry_suite_setup(void);
 void raster_geometry_suite_setup(void)
@@ -584,5 +647,6 @@ void raster_geometry_suite_setup(void)
 	PG_ADD_TEST(suite, test_raster_surface);
 	PG_ADD_TEST(suite, test_raster_perimeter);
 	PG_ADD_TEST(suite, test_raster_pixel_as_polygon);
+	PG_ADD_TEST(suite, test_raster_get_pixel_bilinear);
 }
 
